#include <string>
#include <iostream>
#include <sstream>
#include <iomanip>

using namespace std;

#include <ilcplex/ilocplex.h>
#include <cmath>
#include "fp.h"
#include "AC.h"
#include "glpk.h"

extern "C" {
#include "ACPCx.h"
}

void error_glpk (int stat)
{
	cout << "Finding the analytic center. Error GLPK Solver. " << endl;
	if (stat==GLP_EFAIL) cout << "The problem has no rows/columns" << endl;
	else if (stat==GLP_ENOCVG) cout << "Very show convergence or divergence" << endl;
	else if (stat==GLP_EITLIM) cout << "Iteration limit exceeded" << endl;
	else if (stat==GLP_EINSTAB) cout << "Numerical instability on solving Newtonian system" << endl;
	exit(0);
}

void roundAC(double* InteriorPoint,double* InitialRP, double* ACroundedIntVars)
{
	int entero=0;
	double r;
	for (int i=0;i<nVars;i++) 
	{
		
		if (isInt[i])
		{
			r = floor(relaxedIntVars[entero]+0.5);
			if (stage >1 || isBin[entero])
			{
				if (r<intVars[entero].getLB() || r> intVars[entero].getUB())
					throw string ("relax out of bound");
				
			}
			InteriorPoint[i]=r;
			relaxedIntVars[entero]=InitialRP[i];
			entero++;
		}
		else
		{
			InteriorPoint[i]=relaxedVars[i];
		}
	relaxedVars[i]=InitialRP[i];
	}//end for
}

void findAC(double *x, int solver)
{
	char fileMPS[8]="tmp.mps";
	ACExpr.setConstant(0); //change the f.obj in order to calculate the analytic center (Barrier Method without crossover)
	activeObj.setExpr(ACExpr);
	cplex.setParam(IloCplex::MPSLongNum,0); //short format MPS
        cplex.exportModel(fileMPS);
	
	switch (solver)
	{
		case 1:	// PCx SOLVER
			findACPCx(fileMPS,x);
			break;
		case 2: // GLPK SOLVER (GNU Linear Programming Kit)
			glp_prob *lp;
			lp = glp_create_prob();
			glp_read_mps(lp,GLP_MPS_FILE,NULL,fileMPS);
			if (int stat=glp_interior(lp,NULL) != 0) error_glpk(stat); 
			// reading the current solution
			for (int i=0; i<nVars;i++) x[i] = glp_ipt_col_prim(lp,i+1);
			glp_free_env();
			break;
		case 3: // CPLEX Solver
			IloModel modelAC(env);
			IloObjective ObjectiveAC(env);
			ObjectiveAC.setExpr(ACExpr);
			modelAC.add(ObjectiveAC);
			modelAC.add(vars);
			modelAC.add(rngs);
			IloConversion ACrelaxConv = IloConversion(env, vars, IloNumVar::Float);
			modelAC.add(ACrelaxConv);
			IloCplex cplexAC(modelAC);
			cplexAC.setParam(IloCplex::Threads,1); // CPLEX 12 Multi - Threading
			cplexAC.setParam(IloCplex::RootAlg, IloCplex::Barrier);
			cplexAC.setParam(IloCplex::BarEpComp,1.0e-8);
			cplexAC.setParam(IloCplex::BarCrossAlg, -1);
			cplexAC.setParam(IloCplex::PreInd,0);
			cplexAC.solve();
			cplexAC.getValues(relaxedVars,vars);
			for (int i=0;i<nVars;i++) x[i]=relaxedVars[i];
			ObjectiveAC.end();
			ACExpr.end();
			cplexAC.end();
			modelAC.end();
			break;
	}
}

// Since gamma, find a point between 2 points. Z=gamma*X + (1-gamma)*(Y)
void Point_AC_RP(double gamac,double* analcent)
{
	
	int nint=0;
	for (int i=0;i<nVars;i++)
	{
		relaxedVars[i]= gamac*relaxedVars[i]+ (1-gamac)*analcent[i];
		if (isInt[i])
		{
			relaxedIntVars[nint]=relaxedVars[i];
			nint++;
		}
	}
}
//end addition
