// Feasibility Pump
// Proof-of-concept implementation with ILOG Cplex 9/Concert Technology
//
// Reference papers:
//
// A Feasibility Pump heuristic for general Mixed-Integer Problems
// L. Bertacco, M. Fischetti, A. Lodi. 
// Technical Report OR/05/5 DEIS Universita' di Bologna, 2005
// 
// The Feasibility Pump. 
// M. Fischetti, F. Glover, A. Lodi. 
// Mathematical Programming 2005
// Digital Object Identier (DOI) 10.1007/s10107-004-0570-3.
//
// Livio Bertacco (livio@bertacco.it), July 8th, 2005
// PhD Candidate, University of Padua
// All rights reserved.

#ifndef FP_H
#define FP_H

#include <ostream>
#include <set>
#include <queue>
#include <vector>
#include <ilcplex/ilocplex.h>
#include <ilconcert/ilorandom.h>

using namespace std;

extern double trsmin;
extern int trsrnd;
/* #if 1 */
extern double origobj_norm;             /* norm of the objective function */
extern double alpha;                    /* ratio by which the new obj is combined from distance and original obj */
extern IloExpr dist;
/* #endif */

extern IloEnv env;					// Cplex environment
extern IloRandom rng;               // random number generator
extern IloCplex cplex;				// Cplex solver

extern IloModel model;				// Cplex model
extern IloExpr origObjExpr;			// expression of origObj
extern IloObjective activeObj;		// current obj func.

extern double foconst;						// constant term in the obj function
extern IloNumVarArray vars;			// set of all problem vars
extern IloRangeArray rngs;			// set of all problem constraints
extern IloNumVarArray intVars;		// set of binary variables (subset of vars)
extern IloNumVarArray auxvars;	
extern IloNumVarArray auxintvars;
extern IloNumArray relaxedIntVars;	// value of the int vars if the current relaxation


extern IloNumVar** deltaVars;
extern IloRange** deltaCnstrUB;
extern IloRange** deltaCnstrLB;

extern vector<double> roundedIntVars;			// value of the int vars in the current integer aproximation 
extern vector<double> prevRoundedIntVars;
extern vector<bool> isBin;		



typedef pair<double, int> dipair;	// (sigma, index) pair and comparison function
class dipairCmp {
	public:	bool operator() (const dipair& p1, const dipair& p2) const {return p1.first>p2.first;}
};

extern double epInt;						// integer tolerance
extern int minChange;						// minimum number of flipped var between consecutive integer points
extern int maxIter1;
extern int maxIter2;						// maximum number of iterations
extern int nVars;							// number of variables in the problem
extern int nIntVars;						// number of integer (and binary) variables in the problem
extern int nBinVars;						// number of binary variables in the problem
extern int nDeltaVars;
extern int changed;						// number of variables changed
extern int restarts;
extern int stage;
extern int iter;
extern int s1restarts;

const char* getVarName(IloNumVar var);
void findIntVars();
double round();
bool isCoincident();
void getNextIntegerPt();
void setNewObj();
void getNextIntegerPtS2();
void setNewObjS2();
void cplexSolve();
bool isCoincidentS2();

void restart();
void restartS2();




#endif
